@@ -154,12 +154,16 @@ Load(SV *yaml_sv)
         SvTRUE(GvSV(gv)))
     );
 
-    loader.load_blessed = 1;
+    loader.load_blessed = 0;
     gv = gv_fetchpv("YAML::XS::LoadBlessed", FALSE, SVt_PV);
-    if (SvOK(GvSV(gv))) {
-        if (! SvTRUE(GvSV(gv))) {
-            loader.load_blessed = 0;
-        }
+    if (SvOK(GvSV(gv)) && SvTRUE(GvSV(gv))) {
+        loader.load_blessed = 1;
+    }
+
+    loader.forbid_duplicate_keys = 0;
+    gv = gv_fetchpv("YAML::XS::ForbidDuplicateKeys", FALSE, SVt_PV);
+    if (SvOK(GvSV(gv)) && SvTRUE(GvSV(gv))) {
+        loader.forbid_duplicate_keys = 1;
     }
 
     yaml_str = (const unsigned char *)SvPV_const(yaml_sv, yaml_len);
@@ -352,6 +356,17 @@ load_mapping(perl_yaml_loader_t *loader, char *tag)
         while ((key_node = load_node(loader))) {
             assert(SvPOK(key_node));
             value_node = load_node(loader);
+            if (loader->forbid_duplicate_keys &&
+                hv_exists_ent(hash, key_node, 0)
+            ) {
+                croak(
+                    "%s",
+                    loader_error_msg(
+                        loader,
+                        form("Duplicate key '%s'", SvPV_nolen(key_node))
+                    )
+                );
+            }
             hv_store_ent(
                 hash, sv_2mortal(key_node), value_node, 0
             );
@@ -824,6 +839,7 @@ dump_prewalk(perl_yaml_dumper_t *dumper, SV *node)
 {
     int i, len;
     U32 ref_type;
+    SvGETMAGIC(node);
 
     if (! (SvROK(node) || SvTYPE(node) == SVt_PVGV)) return;
 
@@ -862,11 +878,17 @@ dump_prewalk(perl_yaml_dumper_t *dumper, SV *node)
     else if (ref_type == SVt_PVHV) {
         HV *hash = (HV *)SvRV(node);
         HE *he;
+        SV *key;
+        SV *val;
         hv_iterinit(hash);
+
         while ((he = hv_iternext(hash))) {
-            SV *val = HeVAL(he);
-            if (val)
+            key = hv_iterkeysv(he);
+            he = hv_fetch_ent(hash, key, 0, 0);
+            val = he ? HeVAL(he) : NULL;
+            if (val) {
                 dump_prewalk(dumper, val);
+            }
         }
     }
     else if (ref_type <= SVt_PVNV || ref_type == SVt_PVGV) {
@@ -896,6 +918,7 @@ dump_node(perl_yaml_dumper_t *dumper, SV *node)
     yaml_char_t *tag = NULL;
     const char *class = NULL;
 
+    SvGETMAGIC(node);
     if (SvTYPE(node) == SVt_PVGV) {
         SV **svr;
         tag = (yaml_char_t *)TAG_PERL_PREFIX "glob";
